function plugin(Vue, options) {

    if (!window.axios)
        throw new Error('Axios is required for this plugin');

    if (!options.baseURL)
        throw new Error('Please provided baseURL option');

    if (!options.rootDomainSSO)
        throw new Error('Please provided rootDomainSSO option');

    // Override to authEndpoint
    if (!options.authEndpoint)
        options.authEndpoint = '/auth/login';

    if (!options.key)
        throw new Error('Please provided a key option. The name for the localStorage key');

    /**
     * Create a new Vue instance
     */
    const events = new Vue({

        /**
         * Local data store
         */
        data() {
            return {

                // Token stored
                store: 'unknown',

                // If we set refresh as true by sending a string watch for the token expired prop
                refreshTimeout: false,

                // Iframe context window
                win: false,

                // SSO is ready
                ready: false,

                // Check for ready interval
                checkingInterval: false

            }
        },

        /**
         * When the plugin is created check for the token, if we have it set everything up
         */
        created() {

            // Create the iframe context window
            this.createIframe();

            // Once the window is ready
            window.onload = () => {

                // Listen for any messages coming from the iframe
                this.listenForIframeMessages();

                // Request the token
                this.win.postMessage(JSON.stringify({
                    key: options.key,
                    method: "get"
                }), "*");

            }

        },

        /**
         * Methods
         */
        methods: {

            /**
             * Listen for events from the iframe
             */
            listenForIframeMessages() {

                window.onmessage = (e) => {

                    // If its from our current origin stop here (prevent on localhost)
                    if (e.origin === window.location.origin)
                        return;

                    if (e && e.data) {

                        let message = false;

                        try {
                            message = JSON.parse(e.data);
                        } catch (e) {}

                        if (message) {

                            // Store the token
                            this.$set(this, 'store', message);

                            // Setup our defaults
                            this.setAxiosDefaults(options.baseURL, message.token);

                            // Setup our refresh
                            this.setupRefresh();

                        }

                    }
                    else if (e && e.data === null) {

                        // Store the token
                        this.$set(this, 'store', false);

                    }

                    // We are ready
                    this.ready = true;

                };

            },

            /**
             * Create iframe to communicate with parent
             */
            createIframe() {

                let ifrm = document.createElement("iframe");
                ifrm.setAttribute("src", options.rootDomainSSO);
                ifrm.style.width = "0px";
                ifrm.style.height = "0px";
                ifrm.style.border = "none";
                ifrm.frameborder = "0";
                ifrm.id = "ssoiframe";
                document.body.appendChild(ifrm);

                try {
                    this.win = ifrm.contentWindow;
                } catch (e) {
                    this.win = ifrm.contentWindow;
                }

            },

            /**
             * On auth or created check if we should setup our refreshTimout
             */
            setupRefresh() {

                if (!options.refresh || !this.store.expire)
                    return;

                // Clear the timeout
                this.clearTimeout();

                let expireDate = new Date(this.store.expire);
                let now = new Date();

                // Difference in milliseconds of when the token expires minus 5 mins
                let diff = (expireDate.getTime() - now.getTime()) - (5 * 60) * 1000;

                // If the token is expired already then just logout. All hope is lost
                if (diff <= 0) {
                    this.logout();
                    return;
                }

                // Issue with 32 bit (Max allowed on setTimeout is the following
                else if (diff => 2147483640)
                    diff = 2147483640;

                // Refresh the token before its due to expire
                this.refreshTimeout = setTimeout(() => {

                    // Refresh the token
                    this.refresh();

                }, diff);

            },

            /**
             * Login function
             */
            login(email, pass) {

                return new Promise((resolve, reject) => {

                    if (!email || email === '' || !pass || pass === '') {
                        reject({message: 'Please provided credentials'});
                        return;
                    }

                    // If we have a token then reject this
                    if (this.store) {
                        reject('Already logged in');
                        return;
                    }

                    window.axios.post(options.authEndpoint, {
                        username: email,
                        password: pass
                    }, {baseURL: options.baseURL})
                        .then(response => {

                            // Store the token etc
                            this.setToken(response);

                            // Setup our refresh
                            this.setupRefresh();

                            // Resolve the promise with the data
                            resolve(response.data);

                        })
                        .catch(reject);

                });

            },

            /**
             * If we get a token within the login or refresh run this function
             */
            setToken(response) {

                this.win.postMessage(JSON.stringify({
                    key: options.key,
                    method: "set",
                    data: response.data
                }), "*");

                // Store our token
                this.$set(this, 'store', response.data);

                // Set the defaults
                this.setAxiosDefaults(options.baseURL, response.data.token);

            },

            /**
             * Logout
             */
            logout() {

                return new Promise(resolve => {

                    // Clear the timeout
                    this.clearTimeout();

                    this.win.postMessage(JSON.stringify({
                        key: options.key,
                        method: "remove",
                    }), "*");

                    // Clear our local data
                    this.$nextTick(() => {
                        this.store = false;
                    });

                    // Clear the axios defaults
                    this.clearAxiosDefaults();

                    // Resolve the promise
                    resolve();

                });

            },

            /**
             * Refresh the token
             */
            refresh() {

                // Clear the timeout
                this.clearTimeout();

                return new Promise((resolve, reject) => {

                    window.axios.post(options.refresh, null, {baseURL: options.baseURL})
                        .then(response => {

                            // Store the token etc
                            this.setToken(response);

                            // Setup our refresh
                            this.setupRefresh();

                            // Resolve the promise with the data
                            resolve(response.data);

                        })
                        .catch(() => {

                            // Any issues with refresh lets logout
                            this.logout();

                            reject();

                        });

                });

            },

            /**
             * Set the defaults for Axios
             */
            setAxiosDefaults(url, token) {

                // Default settings for Axios
                window.axios.defaults.baseURL = url;
                window.axios.defaults.headers.common['Authorization'] = 'Bearer ' + token;

            },

            /**
             * Clear out the auth Bearer
             */
            clearAxiosDefaults() {

                // Remove the headers
                window.axios.defaults.headers.common['Authorization'] = null;

            },

            /**
             * Returns if we are logged in or not
             */
            loggedin() {
                return new Promise(resolve => {

                    this.checkingInterval = setInterval(() => {
                        if(this.ready && this.store !== 'unknown') {

                            clearInterval(this.checkingInterval);
                            this.checkingInterval = false;

                            resolve(this.store);

                        }
                    }, 15);

                });
            },

            /**
             * Clear the setTimeout
             */
            clearTimeout() {

                // Make sure we clear it when the component is destroyed
                clearTimeout(this.refreshTimeout);
                this.refreshTimeout = false;

            }

        },

        beforeDestroy() {

            // Clear the timeout
            this.clearTimeout();

            // Clear our checking
            clearInterval(this.checkingInterval);
            this.checkingInterval = false;

        }

    });

    /**
     * Add SSO into the vue prototype
     */
    Object.defineProperty(Vue.prototype, '$SSO', {
        get () {
            return events
        }
    });

}

export default plugin;