import Vue from 'vue/dist/vue';
import test from 'ava';
import Axios from 'axios';
import sinon from 'sinon';

window.axios = Axios;

import SSO from './../dist/SSO.min.js';

let vm;

test.before(t => {
    sinon.stub(window.axios, 'post').resolves({data: {token: 'xxxx', user: 'peter'}});
});

test.after(t => {
    window.axios.post.restore();
});

test.beforeEach(t => {
    vm = new Vue();
});

test.afterEach(t => {
    vm.$destroy();
});

test('It should throw an error if Axios cant be found on the window object', t => {

    window.axios = false;

    t.throws(() => {

        Vue.use(SSO, {
            baseURL: '/api',
            authEndpoint: '/auth/login',
            key: 'my-app',
            refresh: '/auth/refresh_token'
        });

    });

});

test('All methods should be accessible from the root Vue instance', t => {

    window.axios = Axios;

    Vue.use(SSO, {
        baseURL: '/api',
        authEndpoint: '/auth/login',
        key: 'my-app',
        refresh: '/auth/refresh_token'
    });

    // Check global is accessible
    t.is(typeof(vm.$SSO), 'object');

    // Check methods are accessible
    t.is(typeof(vm.$SSO.login), 'function');
    t.is(typeof(vm.$SSO.logout), 'function');
    t.is(typeof(vm.$SSO.loggedin), 'function');

});

function login() {
    return new Promise((resolve, reject) => {
        vm.$SSO.login('user', 'pass')
            .then(() => {
                resolve();
            })
            .catch(reject);
    });
}

test.cb('Check login method stores and sets Axios defaults correctly', t => {

    window.axios = Axios;

    t.false(vm.$SSO._data.store);

    t.is(window.axios.defaults.headers.common['Authorization'], undefined);

    vm.$SSO.login('user', 'pass')
        .then(() => {

            t.is(vm.$SSO._data.store.token, 'xxxx');
            t.is(vm.$SSO._data.store.user, 'peter');

            t.is(window.axios.defaults.headers.common['Authorization'], 'Bearer xxxx');

            t.end();
        });

});

test.cb('Check login, logout & loggedin methods work correctly', t => {

    t.is(vm.$SSO._data.store.token, 'xxxx');
    t.true(vm.$SSO.loggedin());

    vm.$SSO.logout()
        .then(() => {

            t.false(vm.$SSO._data.store);
            t.false(vm.$SSO.loggedin());
            t.is(window.axios.defaults.headers.common['Authorization'], null);

            t.end();

        });

});